#include <stdlib.h>
#include "signed_index.h"
#include "facet_list.h"
#include "combinat.h"
#include "debug.h"
#include "memalloc.h"

static int facet_id=0;

facet_list_p fl_new(int rank, int elements){
  facet_list_p new;
  int j;
  new=malloc(sizeof(facet_list));

  new->head=0;
  new->current=0;
  new->tail=0;
  new->rank=rank;
  new->length=0;
  new->elements=elements;
  new->total_bases=0;
  new->facet_id=malloc(binomial(elements,rank-1)*sizeof(array_index_t));
  for (j=0; j<binomial(elements,rank-1); j++){
    new->facet_id[j]=-1;
  }
    
  return new;
}

static facet_p new_facet(fl_p list,facet_encoding enc, int sign,int size,
			 facet_option_t options){
  facet_p new;
  new=malloc(sizeof(facet)+size*sizeof(natural_t));
  new->encoding=enc;
  new->options=options;
  new->size=size;
  new->sign=sign;
  new->rank=list->rank;
  new->elements=list->elements;
  
  new->facet_id=facet_id++;
  new->next=0;

  if (list->tail)
    list->tail->next=new;
  
  list->tail=new;
  if (list->head == 0){
    list->head=new;
  }

  list->length++;
  list->total_bases+= binomial(size,list->rank)*(list->elements-list->rank);
  list->total_facet_simplices+=binomial(size,list->rank-1);
  return new;
}

void fl_add_bindex(facet_list_p list,int sign,int bindex){
  facet_p new=new_facet(list,basis_index,sign,1,0);

  new->index_set[0]=bindex;
  list->facet_id[bindex]=new->facet_id;

}

void fl_add_vlist(facet_list_p list, int sign,int size, int *index_list,
		  facet_option_t foption){
  facet_p new=new_facet(list,vertex_list,sign,size,foption);
  int d=list->rank-1;

  array_index_t *indices=calloc(d+1,sizeof(int));
  array_index_t *elements=calloc(d+1,sizeof(int));

  bucket_sort(list->elements,size,index_list,new->index_set);

  
  first_kset(size,d,indices);
  do {
    int j;
    int b;
    
    for (j=0; j<d; j++){
      elements[j]=new->index_set[indices[j]-1];
    }
    b=kset_cindex(new->elements,d,elements);
    list->facet_id[b]=new->facet_id;
    err_debug(DB_STARTUP,(HERE,"bdy simplex %d = facet %d\n",b,new->facet_id));

  } while (next_kset(size,d,indices));
  free(indices);
  free(elements);
}

facet_p fl_first(facet_list_p list){
  return list->head;
}

facet_p fl_next(facet_p F){
  return F->next;
}

int fl_length(facet_list_p list){
  return list->length;
}


int fl_sign(facet_p F){

  return F->sign;
  
}

int fl_bindex(facet_p F){
  switch (F->encoding){
  case basis_index:
    return F->index_set[0];
    break;
  case vertex_list:
    if (F->size != (F->rank-1))
      err_fatal(HERE,"attempt to compute basis index of incorrectly sized facet");
    return kset_cindex(F->elements,F->size,F->index_set);
    break;
  default:
    err_fatal(HERE,"facet encoding %d not implemented",F->encoding);
  }
  /*NOTREACHED*/
  return 42;
}

long fl_bases(fl_p list){
  return list->total_bases;
}

long fl_bdy_simplices(fl_p list){
  return list->total_facet_simplices;
}

array_index_t fl_facet_id(fl_p list, array_index_t bindex){
  return list->facet_id[bindex];
}

void fl_output(fl_p list){
  facet_p F;
  for (F=list->head; F; F=F->next){
    if (F->sign !=0){
      fl_output_facet(F);
      printf("\n");
    }
  }
}

extern void fl_output_facet(facet_p F){
  natural_t *pi;
  printf("(");
  switch (F->encoding){
  case basis_index:
    pi=memalloc(sizeof(natural_t)*F->rank);
    output_bindex(F->elements,F->rank,F->index_set[0],pi);
    memfree(pi);
    break;
  case vertex_list:
    output_kset(F->elements,F->size,F->index_set);
    break;
  default:
    err_fatal(HERE,"facet encoding %d not implemented",F->encoding);
  }
  if (F->sign<0){
    printf(",-1");
  } else if (F->options & FO_STELLATE) {
    printf(",stellate");
  } else {
    printf(",1");
  }
  printf(")");
}


void fl_destroy(facet_list_p fl){
  facet_p F,prev;

  F=fl_first(fl);
  
  if (F) {
    prev=F;
    F=F->next;
  }
  while(F){
    free(prev);
    prev=F;
    F=F->next;
  }

  free(fl->facet_id);
  free(fl);
}

    
