#ifndef FACET_LIST_H
#define FACET_LIST_H

#include "oms_types.h"
typedef enum facet_encoding_enum { basis_index=1, vertex_list=2 } facet_encoding;

typedef enum facet_option_t_enum { FO_STELLATE=1 } facet_option_t;

typedef struct facet_struct {
  facet_encoding encoding;
  facet_option_t options;
  int facet_id; /* unique identifier */
  int elements; /* total vertices in polytope */
  int rank;     /* rank of oriented matroid */
  int size;	/* how many vertices on this facet */
  int sign;
  struct facet_struct *next;
  natural_t index_set[1]; /* variable length*/
} facet, *facet_p;

typedef struct fl_struct {
  int length; /* how many facets */
  int elements; /* total vertices in polytope */
  int rank;     /* rank of oriented matroid */
  long total_bases; /* total bases in boundary equations for input facets */
  long total_facet_simplices; /* total bases in boundary equations for input facets */
  facet_p head;
  facet_p current;
  facet_p tail;
  array_index_t *facet_id; /* table from basis index to facet_ident */
} facet_list, *facet_list_p, *fl_p;

facet_list_p fl_new(int rank, int elements);
void fl_add_bindex(facet_list_p list, int sign, int bindex);
void fl_add_vlist(facet_list_p list, int sign, int size, int *index_list,
		  facet_option_t options);
facet_p fl_first(facet_list_p list);
facet_p fl_next(facet_p F);
int fl_length(facet_list_p list);
bool fl_has_next(facet_list_p list);
int fl_sign(facet_p F);
int fl_bindex(facet_p F);
long fl_bases(fl_p list);
long fl_bdy_simplices(fl_p list);
array_index_t fl_facet_id(fl_p list, array_index_t bindex);
extern void fl_output(fl_p list);
extern void fl_output_facet(facet_p F);

#endif
