#ifndef _HYPERLINE_H
#define _HYPERLINE_H

#include "oms_types.h"
#include "statearray.h"
#include "nalloc.h"

#include "hl_slots.h"
#include "basis_equation.h"


#if 0
struct hyperline_cell_struct;
struct hyperline_cfg_struct;
#endif


struct hyperline_cell_struct {
  array_index_t bindex;
  int sign;			/* hyperline element sign, combined 
				   with sorting info */
  int real_sign;		/* just the hyperline sign */
  rank_t key;
  array_index_t row;
  natural_t mu;			/* element index */
  int tau;			/* -1^k where k=#flips to sort into hl. */
  array_index_t rnext,rprev;		/* row linked list */
  array_index_t cnext;		/* coefficient list */
};
typedef struct hyperline_cell_struct hl_cell_t, *hl_cell_p;

struct hyperline_cfg_struct {
  int	   inited;		/* structure is ready to use? */
  sarray_p clist;
  sarray_p left,right;		/* row bounds */
  sarray_p ridge;		/* Does a given hyperline correspond to a ridge? */
  array_index_t rows,cols,rank;
  array_index_t max_rows,max_cols;
  array_index_t coltop;

  size_t colstacksize;
  array_index_t *row_bindex;		/* used to compute indices using 
				   kset_index_complete */
  array_index_t *row_hlindex;		/* where this hyperline goes in the 
				   final lex sorted list of hyperlines */
  array_index_t *hlindex_row;		/* invert the previous table */
  array_index_t *mu_last,*colstack;
  array_index_t *bindex_table; /* mapping from (full dimensional) 
				   basis indices to the maximal 
				   ones stored in the arrangment */
  hl_cell_t cell[1];		/* Variable size */
};


typedef struct hyperline_cfg_struct hyperline_cfg_t, *hyperline_cfg_p,hl_cfg_t,*hl_cfg_p;

/* hyperline.c */
bool hl_set_sign(hl_cfg_p H, sarray_p chirotope, array_index_t bindex, sign_t sign);
void hl_save(hl_cfg_p H);
void hl_restore(hl_cfg_p H);
rank_t hl_key(hl_cfg_p H, array_index_t i);
bool hl_lfeasible(hl_cfg_p H, array_index_t i);
int hl_new_sign(hl_cfg_p H, array_index_t line, sarray_p chirotope);
int hl_new_real_sign(hl_cfg_p H, array_index_t line, sarray_p chirotope);
bool hl_update(na_node_p local, na_node_p global, hl_cfg_p H, sarray_p chirotope);
bool hl_check_boundary(hl_cfg_p H, sarray_p chirotope, array_index_t row, array_index_t k);
array_index_t hl_rows(hl_cfg_p H);
array_index_t hl_cols(hl_cfg_p H);
array_index_t hl_max_cols(hl_cfg_p H);
array_index_t hl_sign(hl_cfg_p H, array_index_t i);
array_index_t hl_tau(hl_cfg_p H, array_index_t i);
array_index_t hl_real_sign(hl_cfg_p H, array_index_t i);
array_index_t hl_index(hl_cfg_p H, array_index_t i);
array_index_t hl_index_inv(hl_cfg_p H, array_index_t bindex);
na_node_p hl_nalloc(na_node_p node, array_index_t slot, int n, int r);
void hl_fixup(na_node_p node, array_index_t slot); /*NA_FUNC*/
void hl_init(hl_cfg_p H, int n, int r, sign_t chi0);
array_index_t hl_left(hl_cfg_p H, array_index_t line);
array_index_t hl_right(hl_cfg_p H, array_index_t line);
void hl_set_right(hl_cfg_p H, array_index_t line, array_index_t val);
void hl_set_left(hl_cfg_p H, array_index_t line, array_index_t val);
array_index_t hl_toright(hl_cfg_p H, array_index_t line, array_index_t k);
array_index_t hl_rnext(hl_cfg_p H, array_index_t k);
array_index_t hl_rprev(hl_cfg_p H, array_index_t k);
array_index_t hl_head(hl_cfg_p H, array_index_t line);
void hl_dump_line(hl_cfg_p H, int line);
void hl_dump(hl_cfg_p H);
bool hl_set_gaps(na_node_p local, na_node_p global, sarray_p chirotope, hl_cfg_p H, array_index_t table_beta, sign_t sign);
bool hl_check_alternation(hl_cfg_p H, array_index_t row);
bool hl_set_ridges(hl_cfg_p H, array_index_t b, be_block_p ridge_table);

#endif
