/**
 * @file   nu_convexity.c
 * @author David Bremner <bremner@unb.ca>
 * @date   Mon Dec 30 18:21:28 2002
 * 
 * @brief  convexity equations for non-uniform (the only currently implemented) * case
 * 
 */


#include "basis_equation.h"
#include "combinat.h"
#include "oms_hook.h"
#include "nalloc.h"
#include "nodedef.h"
#include "globaldef.h"
#include "nu_sign_set.h"
#include "debug.h"

bool nc_traverse(array_index_t bindex,
		      na_node_p local, 
		      na_node_p global,
		      nu_sign_set_p forced,
		      oms_hook force_hook){

  sarray_p chi=na_access(local,SLOT_chirotope);
  be_block_p BE=na_access(global,SLOT_conv_eq);
  sarray_p pos_mask=na_access(local,SLOT_conv_pos_mask);
  sarray_p neg_mask=na_access(local,SLOT_conv_neg_mask);
  sarray_p zero_mask=na_access(local,SLOT_conv_zero_mask);
  
  nu_sign_t chi_bindex=sa_read(chi,bindex);

  array_index_t elem=BE->last[bindex];   

  err_assert( elem >= -1 && elem < BE->rows*BE->cols );
  while ( elem>=0  ){
    array_index_t row,col,row_start;
    int32_t pmask,nmask,zmask;
    nu_sign_t sigma,sigma0;

    row=be_row(BE,elem);
    col=be_col(BE,elem);
    row_start=be_row_start(BE,elem);

    sigma0=sa_read(chi,be_bindex_raw(BE,row_start));
    sigma=sign_mulint(chi_bindex,be_sign_raw(BE,elem));
    pmask=be_update_mask(pos_mask,SIGN_POS,row,col,sigma);
    nmask=be_update_mask(neg_mask,SIGN_NEG,row,col,sigma);
    zmask=be_update_mask(zero_mask,SIGN_ZERO,row,col,sigma);
    if (ERR_DEBUG_MASK & DB_CONVEXITY){
      be_check_mask(global,BE,row,pmask,SIGN_POS,chi);
      be_check_mask(global,BE,row,nmask,SIGN_NEG,chi);
      be_check_mask(global,BE,row,zmask,SIGN_ZERO,chi);
    }
    
    err_debug(DB_CONVEXITY,
	      (HERE,"row=%d, nmask=%x zmask=%x pmask=%x",
	       row,nmask,zmask,pmask));

    if (sign_subset(sigma0,SIGN_NONZERO)){
      if (!be_nonmonotone_row(BE,elem,sigma,pmask,nmask,zmask,forced))
	ERR_RETURN(FALSE);
    }
    elem=BE->eqn[elem].prev;  
  } /* while (elem>0) */

  ERR_RETURN(TRUE);

  

}
